import numpy as np
from numba import jit

__all__ = [
    "pixel_map_c",
    "interp_c",
    "integral_cumsum_c",
    "interp_conserve_c",
    "rebin_weighted_c",
    "midpoint_c",
]


@jit(nopython=True, fastmath=True, error_model="numpy")
def pixel_map_c(in_data, xi, yi, out_data, xo, yo):
    """
    Fast pixel mapping from one image to another

    Parameters
    ----------
    in_data : 2D array
        Input data array

    xi, yi : 1D arrays
        Pixel indices of the second (x) and first (y) array indices

    out_data : 2D array
        Output array will be updated in place ``out_data[yo, xo] = in_data[yi, xi]``

    xo, yo : 1D arrays
        Pixel indices of the second (x) and first (y) array indices

    Returns
    -------
    status : bool

    """
    N = len(xi)
    for i in range(N):
        out_data[yo[i], xo[i]] = in_data[yi[i], xi[i]]

    return True


@jit(nopython=True, fastmath=True, error_model="numpy")
def interp_c(x, xp, fp, extrapolate=0.0, assume_sorted=1):
    """
    Fast linear interpolation: ``f(x) ~ fp(xp)``

    Parameters
    ----------
    x : array-like
        Desired interpolant

    xp, fp : array-like
        Arrays to interpolate

    extrapolate : float
        Value to use where ``x < xp.min()`` or ``x > xp.max()``.

    assume_sorted : int, bool
        The default value of True assumes that the ``x`` array is sorted and
        single-valued, providing a gain in speed.
        ``xp`` is always assumed to be sorted.

    Returns
    -------
    f : array-like
        Interpolated values, same dimensions as ``x``

    """
    N, Np = len(x), len(xp)
    f = np.zeros_like(x)

    i = 0
    j = 0

    ### Handle left extrapolation
    xmin = xp[0]
    if assume_sorted == 1:
        while x[j] < xmin:
            f[j] = extrapolate
            j += 1

    while j < N:
        xval = x[j]
        if assume_sorted == 0:
            if x[j] < xmin:
                f[j] = extrapolate
                j += 1
                continue
            else:
                i = 0

        while (xp[i] < xval) & (i < Np - 1):
            i += 1

        if i == (Np - 1):
            if x[j] != xp[i]:
                f[j] = extrapolate
            else:
                f[j] = fp[i]
            j += 1
            continue

        #### x[i] is now greater than xval because the
        #### expression (x[i]<xval) is false, assuming
        #### that xval < max(x).

        x1 = xp[i - 1]
        x2 = xp[i]
        y1 = fp[i - 1]
        y2 = fp[i]
        out = ((y2 - y1) / (x2 - x1)) * (xval - x1) + y1
        f[j] = out
        j += 1

    return f


@jit(nopython=True, fastmath=True, error_model="numpy")
def integral_cumsum_c(xp, fp, extrapolate=0.0):
    """
    integral_cumsum_c(xp, fp, extrapolate=True)

    Cumulative trapz integration

    """
    #### Implementation of cumsum * dx
    Nxp = len(xp)
    ycumsum = np.zeros_like(fp)

    x0 = xp[0]
    ycumsum[0] = fp[0] * (xp[1] - x0)

    old = ycumsum[0]
    for i in range(1, Nxp):
        x1 = xp[i]
        old += fp[i] * (x1 - x0)
        ycumsum[i] = old
        x0 = x1

    return ycumsum


@jit(nopython=True, fastmath=True, error_model="numpy")
def interp_conserve_c(x, xp, yp, left=0, right=0, integrate=0):
    """
    Interpolate spectrum conserving flux

    Parameters
    ----------
    x : array-like
        Coarse ordinal axis (e.g., wavelength)

    xp, yp : array-like, array-like
        High resolution arrays, (e.g., wavelength and flux density)

    left, right : float, float
        Values to use for extrapolating off the edges of ``xp``

    integrate : bool
        Result is integrated across ``dx``

    Returns
    -------
    outy : array-like
        Interpolated array

    """
    NTEMPL = len(x)
    nxp = len(xp)

    xmid = midpoint_c(x)
    ymid = interp_c(xmid, xp, yp, extrapolate=0.0)

    outy = np.zeros_like(x)

    ######
    # Rebin template grid to master wavelength grid, conserving template flux
    i = 0
    k = 0

    tl0 = xp[0]
    while xmid[k] < tl0:
        outy[k] = left
        k += 1

    for k in range(k, NTEMPL):
        xmk = xmid[k]
        if xmk > xp[nxp - 1]:
            break

        xmk1 = xmid[k + 1]
        ymk = ymid[k]
        ymk1 = ymid[k + 1]

        numsum = 0.0

        ####
        # Go to where xp is greater than the first midpoint
        while (xp[i] < xmk) & (i < nxp):
            i += 1

        tli = xp[i]

        istart = i

        #######
        # First point
        if tli < xmk1:
            h = tli - xmk
            numsum += h * (yp[i] + ymk)
            i += 1

        if i == 0:
            i += 1

        tli = xp[i]
        ypi = yp[i]

        tli1 = xp[i - 1]
        ypi1 = yp[i - 1]

        ######
        # Template points between master grid points
        while (tli < xmk1) & (i < nxp):
            h = tli - tli1
            numsum += h * (ypi + ypi1)

            i += 1
            tli1 = tli
            ypi1 = ypi

            tli = xp[i]
            ypi = yp[i]

        ######
        # If no template points between master grid points, then just use
        # interpolated midpoints
        if i == istart:
            h = xmk1 - xmk
            numsum = h * (ymk1 + ymk)
        else:
            ##### Last point
            if (xmk1 == tli) & (i < nxp):
                h = tli - tli1
                numsum += h * (ypi + ypi1)
            else:
                i -= 1
                h = xmk1 - tli1
                numsum += h * (ymk1 + ypi1)

        outy[k] = numsum * 0.5

        if integrate == 0.0:
            outy[k] /= xmk1 - xmk

    return outy


@jit(nopython=True, fastmath=True, error_model="numpy")
def rebin_weighted_c(x, xp, yp, ye, left=0, right=0, integrate=0, remove_missing=1):
    """
    rebin_weighted_c(x, xp, fp, ep, left=0, right=0, integrate=0)

    Rebin `xp`,`yp` array to the output x array, weighting by `1/ye**2`.
    `xp` can be irregularly spaced.
    """
    NTEMPL = len(x)
    nxp = len(xp)

    outx = np.zeros(NTEMPL - 1, dtype=yp.dtype)
    outy = np.zeros(NTEMPL - 1, dtype=yp.dtype)
    oute = np.zeros(NTEMPL - 1, dtype=yp.dtype)

    ###### Rebin template grid to master wavelength grid weighted by 1/e**2
    i = 0
    k = 0
    while (x[k] < xp[0]) & (k < NTEMPL):
        outy[k] = left
        oute[k] = left
        k += 1

    if k > 0:
        k -= 1

    for k in range(k, NTEMPL - 1):
        if x[k] > xp[nxp - 1]:
            break

        xnumsum = 0.0
        numsum = 0.0
        densum = 0.0

        #### Go to where xp is greater than the x[k]
        while (xp[i] < x[k]) & (i < nxp):
            i += 1

        #### Template points between master grid points
        while (xp[i] <= x[k + 1]) & (i < nxp):
            xnumsum += xp[i] / ye[i] ** 2
            numsum += yp[i] / ye[i] ** 2
            densum += 1.0 / ye[i] ** 2
            # count[i] += 1
            if i == nxp - 1:
                break

            i += 1

        i -= 1

        if densum > 0:
            outx[k] = xnumsum / densum
            outy[k] = numsum / densum
            oute[k] = 1 / densum**0.5

    if remove_missing:
        ok = outx != 0
        return outx[ok], outy[ok], oute[ok]
    else:
        return outx, outy, oute


@jit(nopython=True, fastmath=True, error_model="numpy")
def midpoint_c(x):
    """
    Simple midpoints of array

    Parameters
    ----------
    x : array-like, N
        Target array

    Returns
    -------
    midpoint : array-like, N+1
        Midpoints of ``x``

    """
    N = len(x)
    midpoint = np.zeros(N + 1, dtype=x.dtype)
    midpoint[0] = x[0]
    midpoint[N] = x[N - 1]
    xi1 = x[0]
    for i in range(1, N):
        xi = x[i]
        midpoint[i] = 0.5 * (xi + xi1)
        xi1 = xi

    midpoint[0] = 2 * x[0] - midpoint[1]
    midpoint[N] = 2 * x[N - 1] - midpoint[N - 1]

    return midpoint



#!/usr/bin/env python
# encoding: utf-8
"""
deconvolve.py

GALFIT "deconvolution" as in Szomoru et al.
https://iopscience.iop.org/article/10.1088/2041-8205/714/2/L244

"""

import numpy as np

import astropy.io.fits as pyfits
import astropy.stats
import astropy.table


def test():

    im = pyfits.open('/tmp/gf_out.fits')
    model_hdu = im[2]
    residuals = im[3].data

    _x = galfit_deconvolve(model_hdu, residuals, rms=None)

    model_hdu = _x['model']
    residuals = _x['resid'].data

    imshow_args = {'vmin': -5, 'vmax': 5, 'cmap': 'viridis'}

    _ = galfit_deconvolve(model_hdu, residuals, rms=_x['rms'], imshow_args=imshow_args)


def galfit_deconvolve(model_hdu, residuals, rms=None, mask=None, oversample=8, comp='1', xprof=np.append(0, np.arange(2, 20)), y_func=np.mean, cumul_values=[0.5, 0.8], make_plot=True, imshow_args={'vmin': -5, 'vmax': 5, 'cmap': 'viridis'}, plt_kwargs={'linestyle': 'steps-mid', 'color': 'r', 'alpha': 0.8}, npix=20, psf_offset=[1, 1]):
    """
    Deconvolve an image using a galfit Sersic model

    `model_hdu` is the HDU output from GALFIT containing the model and model
    parameters (e.g, out.fits[2]).

    `residuals` are the fit residuals (out.fits[3].data)

    `psf_offset` is the offset of the PSF relative to the central pixel of
    the PSF array.

    Returns:

        R = radii evaluated at pixel positions for the ellipse parameters
        phi = position angle around the ellipse, radians
        prof = 2D array of unconvolved Sersic profile
        params = Galfit params
        tab = table of the 1D profile
        fig = optional figure if `make_plot=True`

    """
    import matplotlib.pyplot as plt

    #model_hdu = _x['model']
    #residuals = _x['resid']

    _h = model_hdu.header
    shape = residuals.shape

    if mask is None:
        mask = np.isfinite(residuals)

    # Ellipse parameters and sersic profile
    R, phi, prof, params = sersic_profile(shape, oversample=oversample,
                                          gf_header=_h, comp=comp,
                                          psf_offset=psf_offset)

    so = np.argsort(R[mask].flatten())
    Rso = R[mask].flatten()[so]

    data = residuals + model_hdu.data

    if '1_SKY' in _h:
        sky = gf_header_key(_h, '1_SKY')
    else:
        sky = 0.

    ##########
    # 1D averaged profiles

    integ = False

    # Original data
    _, ydata, _, _ = running_median(Rso,
                (data)[mask].flatten()[so].astype(np.float)-sky,
                bins=xprof, use_median=False, y_func=y_func, integrate=integ)
    
    # Convolved model
    _, ymodel, _, _ = running_median(Rso,
                (model_hdu.data)[mask].flatten()[so].astype(np.float),
                bins=xprof, use_median=False, y_func=y_func, integrate=integ)
    
    # Sersic profile
    _, yprof, _, _ = running_median(Rso,
                (prof)[mask].flatten()[so].astype(np.float),
                bins=xprof, use_median=False, y_func=y_func, integrate=integ)

    # Sersic + residuals
    xm, ydeconv, ys, yn = running_median(Rso,
                (prof + residuals)[mask].flatten()[so].astype(np.float),
                bins=xprof, use_median=False, y_func=y_func, integrate=integ)

    # Sum for total normalizatioin
    _, ydeconv_sum, _, _ = running_median(Rso,
                (prof + residuals)[mask].flatten()[so].astype(np.float),
                bins=xprof, use_median=False, y_func=np.sum, integrate=False)

    # Variance
    if rms is not None:
        xmx, yv, yvs, yn = running_median(Rso,
                rms[mask].flatten()[so].astype(np.float)**2,
                bins=xprof, use_median=False, y_func=np.sum)

        yrms = np.sqrt(yv)/yn
        im_norm = rms
        
        # weighted
        xmx, ynum_model, yvs, yn = running_median(Rso,
                (model_hdu.data/rms**2)[mask].flatten()[so].astype(np.float),
                bins=xprof, use_median=False, y_func=np.sum)

        xmx, ynum, yvs, yn = running_median(Rso,
                (data/rms**2)[mask].flatten()[so].astype(np.float),
                bins=xprof, use_median=False, y_func=np.sum)
                
        xmx, yden, yvs, yn = running_median(Rso,
                (1./rms**2)[mask].flatten()[so].astype(np.float),
                bins=xprof, use_median=False, y_func=np.sum)
        
        yweight = ynum/yden
        yweight_model = ynum_model/yden
        yweight_err = 1./np.sqrt(yden)
        
    else:
        yrms = ys
        im_norm = 1
        yweight = None
        
    dx = np.diff(xprof)
    xpix = xprof[1:]-dx/2.

    if False:
        # Scale
        yscl = 1.

        #yscl = prof[msk].sum()/np.trapz(2*np.pi*yprof*xpix, xpix)
        yprof *= yscl
        ydeconv *= yscl
        yrms *= yscl

    # Interpolate Radii from the cumulative flux distribution
    cumul_flux = np.cumsum((prof + residuals)[mask].flatten()[so])

    #total = cumflux.max()
    total = ydeconv_sum.sum()

    Rcumul = np.interp(cumul_values, cumul_flux/total, Rso)

    tab = astropy.table.Table()
    tab['xpix'] = xpix
    tab['yprofile'] = yprof
    tab['ydeconv'] = ydeconv
    tab['yrms'] = yrms
    tab['ydata'] = ydata
    tab['ymodel'] = ymodel
    
    if yweight is not None:
        tab['yweight'] = yweight
        tab['yweight_model'] = yweight_model
        tab['yweight_err'] = yweight_err
        
    tab.meta['total_flux'] = total
    for k in params:
        tab.meta[k] = params[k]

    for i, r in enumerate(cumul_values):
        tab.meta['R{0}'.format(int(r*100))] = Rcumul[i]

    params['Rcumul'] = Rcumul

    if make_plot:

        fig = plt.figure(figsize=(12, 3))
        ax = fig.add_subplot(141)
        ax2 = fig.add_subplot(142)
        ax3 = fig.add_subplot(143)

        ax4 = fig.add_subplot(144)
        ax4.loglog()
        ax4.grid()

        xc = gf_header_key(_h, comp+'_XC')
        yc = gf_header_key(_h, comp+'_YC')

        ax.imshow(data/im_norm*mask, **imshow_args)
        ax2.imshow(residuals/im_norm*mask, **imshow_args)
        ax3.imshow(prof/im_norm*mask, **imshow_args)

        label = 'Re={re:.2f}, n={n:.1f}'.format(**params)
        pl = ax4.plot(xpix, yprof, label=label, **plt_kwargs)

        ax4.errorbar(xpix, ydeconv, yrms, linestyle='None',
                     color=pl[0].get_color(), alpha=0.5, marker='.')

        ax4.vlines(params['re'], 1e-10, 1e10, color=pl[0].get_color(), alpha=0.5)
        ax4.vlines(Rcumul, 1e-10, 1e10, color=pl[0].get_color(), alpha=0.5, linewidth=3, linestyle='--')

        ax4.scatter(Rso,
                (prof + residuals)[mask].flatten()[so].astype(np.float)-sky,
                marker='.', color='k', alpha=0.1, zorder=-1000)

        # ax4.scatter(Rso,
        #        (data)[mask].flatten()[so].astype(np.float)-sky,
        #        marker='.', color='r', alpha=0.1, zorder=-1000)

        ax4.legend()

        valid = np.isfinite(ydeconv) & np.isfinite(yprof)

        print(ydeconv[valid].min(), ydeconv[valid].max())

        try:
            ax4.set_ylim(np.maximum(0.5*yprof[valid].min(), 1e-5*ydeconv[valid].max()),
                     2*ydeconv[valid].max())
        except:
            pass

        ax4.set_xlim(0.05, xprof.max()+2)

        for a in [ax, ax2, ax3]:
            a.set_xlim(xc-1-npix, xc-1+npix)
            a.set_ylim(yc-1-npix, yc-1+npix)
            a.set_xticklabels([])
            a.set_yticklabels([])

            # Show ellipses on images
            a.plot(*r_ellipse(Rcumul[0], psf_offset=psf_offset, **params),
                   color='w', alpha=0.9, linewidth=2)
            a.plot(*r_ellipse(xpix.max(), psf_offset=psf_offset, **params),
                   color='w', alpha=0.9, linewidth=2)

            a.plot(*r_ellipse(Rcumul[0], psf_offset=psf_offset, **params),
                   color=pl[0].get_color(), alpha=0.5)
            a.plot(*r_ellipse(xpix.max(), psf_offset=psf_offset, **params),
                   color=pl[0].get_color(), alpha=0.5, linestyle='--')

        fig.tight_layout(pad=0.1)

    else:
        fig = None

    return R, phi, prof, params, tab, fig


def get_kappa(n, **kwargs):
    """
    Compute the Kappa parameter for a given n index as in the Galfit
    definition
    """
    from scipy.optimize import root
    x0 = 2*n-0.33
    args = (n)
    k = root(kappa_func, x0, args=args, **kwargs)
    return k.x[0]


def kappa_func(kappa, n):
    """
    Function for getting Sersic kappa
    """
    from scipy.special import gamma, gammainc
    f = gamma(2*n)-2*gammainc(2*n, kappa)*gamma(2*n)
    return f


def Rc(c0):
    """
    Shape parameter
    """
    from scipy.special import beta

    return np.pi*(c0+2)/(4*beta(1./(c0+2), 1+1./(c0+2)))


def sigma_e(re, n, q, Ftot=1., c0=0.):
    """
    Surface brightess at the effective radius, re, given total flux
    """
    from scipy.special import gamma
    kap = get_kappa(n)
    return Ftot/(2*np.pi*re**2*np.exp(kap)*n*kap**(-2*n)*gamma(2*n)*q/Rc(c0)), kap


def gf_header_key(header, key='2_XC'):
    """
    Get a header keyword from a GALFIT header, which may have [] and *
    in the keyword value
    """
    return float(header[key].split()[0].strip('[]').strip('*').strip('{').strip('}'))


def sersic_profile(shape, mag=20., xc=[0., 0.], n=1., q=0.5, pa=0, re=1., ZP=26., gf_header=None, comp='2', verbose=True, oversample=8, psf_offset=[1, 1]):
    """
    Generate a Sersic profile with Galfit parameters within a defined image
    shape.

    Specify the parameters individually or provide a GALFIT model header

    gf_header: FITS header of a GALFIT output model
    comp: Number of the object in the GALFIT model

    """
    import scipy.ndimage as nd

    if gf_header is not None:
        xx = gf_header_key(gf_header, comp+'_XC')
        yy = gf_header_key(gf_header, comp+'_YC')
        xc = np.array([xx, yy])
        mag = gf_header_key(gf_header, comp+'_MAG')
        if comp+'_N' in gf_header:
            n = gf_header_key(gf_header, comp+'_N')
            q = gf_header_key(gf_header, comp+'_AR')
            pa = gf_header_key(gf_header, comp+'_PA')
            re = gf_header_key(gf_header, comp+'_RE')
        else:
            n = 1.
            q = 1.
            pa = 0.
            re = 0.01

        if verbose:
            print(f'xc:{xc}, q:{q}, pa:{pa}, n:{n}')

        if 'MAGZPT' in gf_header:
            ZP = gf_header['MAGZPT']

    params = {'mag': mag, 'xc': xc, 're': re, 'n': n, 'q': q, 'pa': pa}

    sigm, kap = sigma_e(re, n, q)
    norm = sigm*10**(-0.4*(mag-ZP))
    R, x, phi = pix_to_r(shape, verbose=verbose, psf_offset=psf_offset,
                         **params)

    if oversample > 1:
        Ro, x, phio = pix_to_r(shape, oversample=oversample, verbose=verbose,
                              psf_offset=psf_offset, **params)

        sersic_large = norm*np.exp(-kap*((Ro/re)**(1./n)-1))
        kern = np.ones((oversample, oversample))/oversample**2
        sersic_profile = nd.convolve(sersic_large, kern)[oversample//2-1::oversample, oversample//2-1::oversample]
    else:
        sersic_profile = norm*np.exp(-kap*((R/re)**(1./n)-1))

    return R, phi, sersic_profile, params


def r_ellipse(radius=5, xc=[0., 0.], q=0.5, pa=0, re=1., gf_header=None, comp='2', verbose=True, nstep=256, psf_offset=[1, 1], **kwargs):
    """
    Make x, y coordinates given ellipse parameters
    """
    if gf_header is not None:
        xx = gf_header_key(gf_header, comp+'_XC')
        yy = gf_header_key(gf_header, comp+'_YC')
        xc = np.array([xx, yy])
        mag = gf_header_key(gf_header, comp+'_MAG')

        if comp+'_N' in gf_header:
            n = gf_header_key(gf_header, comp+'_N')
            q = gf_header_key(gf_header, comp+'_AR')
            pa = gf_header_key(gf_header, comp+'_PA')
            re = gf_header_key(gf_header, comp+'_RE')
        else:
            n = 1.
            q = 1.
            pa = 0.
            re = 0.01

        if verbose:
            print(f'xc:{xc}, q:{q}, pa:{pa}')

    phi = np.linspace(0, 2*np.pi, nstep)
    xp = np.array([np.cos(phi), q*np.sin(phi)]).T*radius

    theta = -(np.pi/2 + pa/180*np.pi)  # + np.pi

    _rot = np.array([[np.cos(theta), -np.sin(theta)], [np.sin(theta), np.cos(theta)]])

    x0 = xp.dot(_rot) + np.atleast_1d(xc)
    xell, yell = (x0 - 1 - np.atleast_1d(psf_offset)).T

    return xell, yell


def pix_to_r(shape, xc=[0., 0.], q=0.5, pa=0, re=1., gf_header=None, comp='2', verbose=True, oversample=1, psf_offset=[1, 1], **kwargs):
    """
    Return an array of ellipse radii for a pixel grid and ellipse parameters
    """

    if oversample > 1:
        xarr = np.arange(-0.5, shape[1]-0.5, 1/oversample)
        yarr = np.arange(-0.5, shape[0]-0.5, 1/oversample)
    else:
        xarr = np.arange(0, shape[1], 1/oversample)
        yarr = np.arange(0, shape[0], 1/oversample)

    x0 = np.array(np.meshgrid(xarr, yarr)).reshape((2, -1)).T + 1
    x0 += np.atleast_1d(psf_offset)

    if gf_header is not None:
        xx = gf_header_key(gf_header, comp+'_XC')
        yy = gf_header_key(gf_header, comp+'_YC')
        xc = np.array([xx, yy])
        mag = gf_header_key(gf_header, comp+'_MAG')

        if comp+'_N' in gf_header:
            n = gf_header_key(gf_header, comp+'_N')
            q = gf_header_key(gf_header, comp+'_AR')
            pa = gf_header_key(gf_header, comp+'_PA')
            re = gf_header_key(gf_header, comp+'_RE')
        else:
            n = 1.
            q = 1.
            pa = 0.
            re = 0.01

        if verbose:
            print(f'xc:{xc}, q:{q}, pa:{pa}')

    theta = (np.pi/2 + pa/180*np.pi)  # + np.pi

    _rot = np.array([[np.cos(theta), -np.sin(theta)], [np.sin(theta), np.cos(theta)]])

    xp = (x0 - np.atleast_1d(xc)).dot(_rot)

    outshape = [s*oversample for s in shape]

    R = np.sqrt(((xp/np.array([1, q]))**2).sum(axis=1)).reshape(outshape)

    phi = np.arctan2(xp[:, 1], xp[:, 0]).reshape(outshape)

    return R, re, phi


def running_median(xi, yi, NBIN=10, use_median=True, use_nmad=True, reverse=False, bins=None, x_func=astropy.stats.biweight_location, y_func=astropy.stats.biweight_location, std_func=astropy.stats.biweight_midvariance, integrate=False):
    """
    Running median/biweight/nmad
    """

    NPER = xi.size // NBIN
    if bins is None:
        so = np.argsort(xi)
        if reverse:
            so = so[::-1]

        bx = np.linspace(0, len(xi), NBIN+1)
        bins = np.interp(bx, np.arange(len(xi)), xi[so])
        if reverse:
            bins = bins[::-1]

    NBIN = len(bins)-1

    xm = np.arange(NBIN)*1.
    xs = xm*0
    ym = xm*0
    ys = xm*0
    N = np.arange(NBIN)

    if use_median:
        y_func = np.median

    if use_nmad:
        std_func = astropy.stats.mad_std

    for i in range(NBIN):
        in_bin = (xi > bins[i]) & (xi <= bins[i+1])
        N[i] = in_bin.sum()  # N[i] = xi[so][idx+NPER*i].size

        if integrate:
            xso = np.argsort(xi[in_bin])
            ma = xi[in_bin].max()
            mi = xi[in_bin].min()
            xm[i] = (ma+mi)/2.
            dx = (ma-mi)
            ym[i] = np.trapz(yi[in_bin][xso], xi[in_bin][xso])/dx
        else:
            xm[i] = x_func(xi[in_bin])
            ym[i] = y_func(yi[in_bin])

        ys[i] = std_func(yi[in_bin])

    return xm, ym, ys, N
